@@ -645,15 +645,16 @@ There is a bug tracker at .
 improvement to this module, and who want to share the change with
 other people by adding it to the public version of this module.)
 
-If you want to alter this module, please note very carefully that the
-distributed files are not actually the source code of the module. If
-you wish to obtain the source code of Image::PNG, please access
-http://www.lemoda.net/projects/image-png/repo/ using the "git" source control system.
+If you want to alter this module, note very carefully that the
+distributed files are not actually the source code of the module. The
+source code lives in the "tmpl" directory of the distribution and the
+distribution is created via scripts.
 
 The original files of this distribution make very heavy use of the
 L<Template> Perl module in order to cut down the amount of repetitive
 stuff which needs to be put into various source and documentation
-files. If you plan to alter the C file, you also need a program called
-"cfunctions" which you can download from sourceforge.
+files. If you plan to alter the C file, you may also need a program
+called "cfunctions" which you can download from sourceforge, which
+generates the header file perl-libpng.h from perl-libpng.c.
 
 
