@@ -512,7 +512,7 @@ perl_png_set_text_from_hash (perl_libpng_t * png, png_text * text_out, HV * chun
         SV ** field_sv_ptr = hv_fetch (chunk, #field,           \
                                        strlen (#field), 0);     \
         if (! field_sv_ptr) {                                   \
-            fprintf (stderr, "You don't have no %s sucker.\n",  \
+            fprintf (stderr, "Field %s is missing.\n",          \
                      #field);                                   \
             return 0;                                           \
         }                                                       \
@@ -544,7 +544,7 @@ perl_png_set_text_from_hash (perl_libpng_t * png, png_text * text_out, HV * chun
         SV ** field_sv_ptr = hv_fetch (chunk, #field,           \
                                        strlen (#field), 0);     \
         if (! field_sv_ptr) {                                   \
-            fprintf (stderr, "You don't have no %s sucker.\n",  \
+            fprintf (stderr, "Field %s is missing.\n",          \
                      #field);                                   \
             field = "";                                         \
         }                                                       \
@@ -715,7 +715,10 @@ perl_png_get_tIME (perl_libpng_t * png)
 void
 perl_png_set_tIME (perl_libpng_t * png, SV * input_time)
 {
-    png_time mod_time = {0};
+    /* The PNG month and day fields shouldn't be equal to zero. 
+       See PNG specification "4.2.4.6. tIME Image last-modification time"
+    */
+    png_time mod_time = {0,1,1,0,0,0};
     SV * ref;
 
     ref = SvRV(input_time);
@@ -730,9 +733,6 @@ perl_png_set_tIME (perl_libpng_t * png, SV * input_time)
                 MESSAGE ("OK for %s\n", #field);                        \
                 mod_time.field = SvIV (field_sv);                       \
             }                                                           \
-            else {                                                      \
-                mod_time.field = 0;                                     \
-            }                                                           \
         }
         SET_TIME(year);
         SET_TIME(month);
@@ -895,7 +895,7 @@ perl_png_write_to_scalar (perl_libpng_t * png, int transforms)
 void
 perl_png_write_png (perl_libpng_t * png, int transforms)
 {
-    MESSAGE ("Trying to write your crap.");
+    MESSAGE ("Trying to write a PNG.");
     png_write_png (pngi, transforms, UNUSED_ZERO_ARG);
 }
 
@@ -1142,6 +1142,21 @@ static HV * perl_png_color_16_to_hv (png_color_16p colour)
     return perl_colour;
 }
 
+/* Turn a hash into the colour values of a pointer to a png_color_16
+   structure. */
+
+static void perl_png_hv_to_color_16 (HV * perl_colour, png_color_16p colour)
+{
+#define PERL_COLOUR(x) \
+    HASH_FETCH_IV_MEMBER (perl_colour, x, colour)
+    PERL_COLOUR(index);
+    PERL_COLOUR(red);
+    PERL_COLOUR(green);
+    PERL_COLOUR(blue);
+    PERL_COLOUR(gray);
+#undef PERL_COLOUR
+}
+
 /*   ___  _   _                      _                 _        
     / _ \| |_| |__   ___ _ __    ___| |__  _   _ _ __ | | _____ 
    | | | | __| '_ \ / _ \ '__|  / __| '_ \| | | | '_ \| |/ / __|
@@ -1163,22 +1178,15 @@ SV * perl_png_get_bKGD (perl_libpng_t * png)
     UNDEF;
 }
 
+/* Set the bKGD chunk of the image from values in a hash
+   "bKGD". Values not set in the hash are set to zero. */
+
 void perl_png_set_bKGD (perl_libpng_t * png, HV * bKGD)
 {
-}
-
-SV * perl_png_get_cHRM (perl_libpng_t * png)
-{
-    if (VALID (cHRM)) {
-        HV * ice;
-        ice = newHV ();
-        return newRV_inc ((SV *) ice);
-    }
-    UNDEF;
-}
-
-void perl_png_set_cHRM (perl_libpng_t * png, HV * cHRM)
-{
+    /* Default is all zeros. */
+    png_color_16 background = {0};
+    perl_png_hv_to_color_16 (bKGD, & background);
+    png_set_bKGD (pngi, & background);
 }
 
 SV * perl_png_get_pCAL (perl_libpng_t * png)
@@ -1270,11 +1278,17 @@ void perl_png_set_sCAL (perl_libpng_t * png, HV * sCAL)
 {
 }
 
+/* Should this be a hash value or an array? */
+
+/* "4.2.4.3. sBIT Significant bits" */
+
 SV * perl_png_get_sBIT (perl_libpng_t * png)
 {
     if (VALID (sBIT)) {
         HV * sig_bit;
+        png_color_8 colours;
         sig_bit = newHV ();
+         HASH_STORE_IV_MEMBER (sig_bit, red, colours); HASH_STORE_IV_MEMBER (sig_bit, green, colours); HASH_STORE_IV_MEMBER (sig_bit, blue, colours); HASH_STORE_IV_MEMBER (sig_bit, gray, colours); HASH_STORE_IV_MEMBER (sig_bit, alpha, colours);
         return newRV_inc ((SV *) sig_bit);
     }
     UNDEF;
@@ -1282,6 +1296,9 @@ SV * perl_png_get_sBIT (perl_libpng_t * png)
 
 void perl_png_set_sBIT (perl_libpng_t * png, HV * sBIT)
 {
+    png_color_8 colours;
+     HASH_FETCH_IV_MEMBER (sBIT,red,(&colours)); HASH_FETCH_IV_MEMBER (sBIT,green,(&colours)); HASH_FETCH_IV_MEMBER (sBIT,blue,(&colours)); HASH_FETCH_IV_MEMBER (sBIT,gray,(&colours)); HASH_FETCH_IV_MEMBER (sBIT,alpha,(&colours));
+    png_set_sBIT (pngi, & colours);
 }
 
 SV * perl_png_get_oFFs (perl_libpng_t * png)
@@ -1289,7 +1306,14 @@ SV * perl_png_get_oFFs (perl_libpng_t * png)
     if (VALID (oFFs)) {
         HV * offset;
         
+        png_uint_32 x_offset;
+        png_uint_32 y_offset;
+        int unit_type;
         offset = newHV ();
+        png_get_oFFs (pngi, & x_offset, & y_offset, & unit_type);
+        HASH_STORE_IV (offset, x_offset);
+        HASH_STORE_IV (offset, y_offset);
+        HASH_STORE_IV (offset, unit_type);
         return newRV_inc ((SV *) offset);
     }
     UNDEF;
@@ -1297,13 +1321,27 @@ SV * perl_png_get_oFFs (perl_libpng_t * png)
  
 void perl_png_set_oFFs (perl_libpng_t * png, HV * oFFs)
 {
+    png_uint_32 x_offset;
+    png_uint_32 y_offset;
+    int unit_type;
+    HASH_FETCH_IV (oFFs, x_offset);
+    HASH_FETCH_IV (oFFs, y_offset);
+    HASH_FETCH_IV (oFFs, unit_type);
+    png_set_oFFs (pngi, x_offset, y_offset, unit_type);
 }
 
 SV * perl_png_get_pHYs (perl_libpng_t * png)
 {
     if (VALID (pHYs)) {
+        png_uint_32 res_x;
+        png_uint_32 res_y;
+        int unit_type;
         HV * phys;
         phys = newHV ();
+        png_get_pHYs (pngi, & res_x, & res_y, & unit_type);
+        HASH_STORE_IV (phys, res_x);
+        HASH_STORE_IV (phys, res_y);
+        HASH_STORE_IV (phys, unit_type);
         return newRV_inc ((SV *) phys);
     }
     UNDEF;
@@ -1311,6 +1349,13 @@ SV * perl_png_get_pHYs (perl_libpng_t * png)
 
 void perl_png_set_pHYs (perl_libpng_t * png, HV * pHYs)
 {
+    png_uint_32 res_x;
+    png_uint_32 res_y;
+    int unit_type;
+    HASH_FETCH_IV (pHYs, res_x);
+    HASH_FETCH_IV (pHYs, res_y);
+    HASH_FETCH_IV (pHYs, unit_type);
+    png_set_pHYs (pngi, res_x, res_y, unit_type);
 }
 
 /* Get the transparency information for a paletted image. */
@@ -1628,6 +1673,8 @@ SV * perl_png_get_unknown_chunks (perl_libpng_t * png)
     }
 }
 
+/* Set private chunks in the PNG. */
+
 void perl_png_set_unknown_chunks (perl_libpng_t * png, AV * chunk_list)
 {
     /* n_chunks is the number of chunks the user proposes to add to
@@ -1806,6 +1853,87 @@ perl_png_set_strip_16 (perl_libpng_t * png)
     png_set_strip_16 (png->png);
 }
 
+SV * perl_png_get_cHRM (perl_libpng_t * png)
+{
+    if (VALID (cHRM)) {
+        HV * ice;
+        double white_x;
+        double white_y;
+        double red_x;
+        double red_y;
+        double green_x;
+        double green_y;
+        double blue_x;
+        double blue_y;
+        png_get_cHRM (pngi , & white_x, & white_y, & red_x, & red_y, & green_x, & green_y, & blue_x, & blue_y);
+        ice = newHV ();
+        hv_store (ice, "white_x", strlen ("white_x"),
+                   newSVnv (white_x), 0);
+        hv_store (ice, "white_y", strlen ("white_y"),
+                   newSVnv (white_y), 0);
+        hv_store (ice, "red_x", strlen ("red_x"),
+                   newSVnv (red_x), 0);
+        hv_store (ice, "red_y", strlen ("red_y"),
+                   newSVnv (red_y), 0);
+        hv_store (ice, "green_x", strlen ("green_x"),
+                   newSVnv (green_x), 0);
+        hv_store (ice, "green_y", strlen ("green_y"),
+                   newSVnv (green_y), 0);
+        hv_store (ice, "blue_x", strlen ("blue_x"),
+                   newSVnv (blue_x), 0);
+        hv_store (ice, "blue_y", strlen ("blue_y"),
+                   newSVnv (blue_y), 0);
+        return newRV_inc ((SV *) ice);
+    }
+    UNDEF;
+}
+
+void perl_png_set_cHRM (perl_libpng_t * png, HV * cHRM)
+{
+    SV ** key_sv_ptr;
+    double white_x = 0.0;
+    double white_y = 0.0;
+    double red_x = 0.0;
+    double red_y = 0.0;
+    double green_x = 0.0;
+    double green_y = 0.0;
+    double blue_x = 0.0;
+    double blue_y = 0.0;
+    key_sv_ptr = hv_fetch (cHRM, "white_x", strlen ("white_x"), 0);
+    if (key_sv_ptr) {
+        white_x = SvNV (* key_sv_ptr);
+    }
+    key_sv_ptr = hv_fetch (cHRM, "white_y", strlen ("white_y"), 0);
+    if (key_sv_ptr) {
+        white_y = SvNV (* key_sv_ptr);
+    }
+    key_sv_ptr = hv_fetch (cHRM, "red_x", strlen ("red_x"), 0);
+    if (key_sv_ptr) {
+        red_x = SvNV (* key_sv_ptr);
+    }
+    key_sv_ptr = hv_fetch (cHRM, "red_y", strlen ("red_y"), 0);
+    if (key_sv_ptr) {
+        red_y = SvNV (* key_sv_ptr);
+    }
+    key_sv_ptr = hv_fetch (cHRM, "green_x", strlen ("green_x"), 0);
+    if (key_sv_ptr) {
+        green_x = SvNV (* key_sv_ptr);
+    }
+    key_sv_ptr = hv_fetch (cHRM, "green_y", strlen ("green_y"), 0);
+    if (key_sv_ptr) {
+        green_y = SvNV (* key_sv_ptr);
+    }
+    key_sv_ptr = hv_fetch (cHRM, "blue_x", strlen ("blue_x"), 0);
+    if (key_sv_ptr) {
+        blue_x = SvNV (* key_sv_ptr);
+    }
+    key_sv_ptr = hv_fetch (cHRM, "blue_y", strlen ("blue_y"), 0);
+    if (key_sv_ptr) {
+        blue_y = SvNV (* key_sv_ptr);
+    }
+    png_set_cHRM (pngi, white_x, white_y, red_x, red_y, green_x, green_y, blue_x, blue_y);
+}
+
 /*
    Local Variables:
    mode: c
